@@ -8,8 +8,7 @@
 # @Last modified by: José Sánchez-Gallego
 # @Last Modified time: 2017-12-05 12:19:32
 
-
-__all__ = ['JaegerError', 'JaegerCANError', 'JaegerUserWarning']
+import inspect
 
 
 class JaegerError(Exception):
@@ -51,6 +50,54 @@ class JaegerCANError(JaegerError):
         super(JaegerCANError, self).__init__(message)
 
 
+class PositionerError(JaegerError):
+    """Exception class for positioner-related errors."""
+
+    def __init__(self, message=None, positioner=None):
+
+        if message is None:
+            message = ''
+
+        if positioner is not None:
+            pid = positioner.positioner_id
+        else:
+            stack = inspect.stack()
+            f_locals = stack[1][0].f_locals
+
+            if 'self' in f_locals:
+                pid = f_locals['self'].positioner_id
+            else:
+                pid = 'UNKNOWN'
+
+        message = f'Positioner {pid}: {message}'
+
+        super(PositionerError, self).__init__(message)
+
+
+class CommandError(JaegerError):
+    """Exception class for command-related errors."""
+
+    def __init__(self, message=None, command=None):
+
+        if message is None:
+            message = ''
+
+        if command is None:
+            stack = inspect.stack()
+            f_locals = stack[1][0].f_locals
+
+            if 'self' in f_locals:
+                command = f_locals['self']
+
+        if command:
+            c_name = command.name
+            pid = command.positioner_id
+            c_uid = command.command_uid
+            message = f'({c_name}, {pid}, {c_uid!s}): {message}'
+
+        super(CommandError, self).__init__(message)
+
+
 class JaegerMissingDependency(JaegerError):
     """A custom exception for missing dependencies."""
     pass
