@@ -18,9 +18,10 @@ from drift import Drift, DriftError
 from jaeger import config, log, start_file_loggers
 from jaeger.can import JaegerCAN
 from jaeger.commands import Command, CommandID, send_trajectory
-from jaeger.exceptions import FPSLockedError, JaegerUserWarning, TrajectoryError
+from jaeger.exceptions import (FPSLockedError, JaegerError,
+                               JaegerUserWarning, TrajectoryError)
 from jaeger.positioner import Positioner
-from jaeger.utils import Poller, PollerList, bytes_to_int, get_qa_database
+from jaeger.utils import Poller, PollerList, bytes_to_int
 
 
 # try:
@@ -132,7 +133,6 @@ class BaseFPS(dict):
 
             for row in data:
                 if row['type'].lower() == 'fiducial':
-                    print('f')
                     continue
                 self.add_positioner(row['id'], centre=(row['x'], row['y']))
 
@@ -171,31 +171,13 @@ class BaseFPS(dict):
         """Adds a new positioner to the list, and checks for duplicates."""
 
         if positioner_id in self.positioners:
-            raise ValueError(f'{self._class_name}: there is already a '
-                             f'positioner in the list with positioner_id '
-                             f'{positioner_id}.')
+            raise JaegerError(f'{self._class_name}: there is already a '
+                              f'positioner in the list with positioner_id '
+                              f'{positioner_id}.')
 
         self.positioners[positioner_id] = self._positioner_class(positioner_id, self,
                                                                  centre=centre)
 
-        if self.qa_db:
-
-            Positioner = self.qa_db.models['Positioner']
-
-            # Check if the positioner exists.
-            db_pos = Positioner.select().filter(Positioner.id == positioner_id).first()
-
-            if not db_pos:
-                new = True
-                db_pos = Positioner(id=positioner_id)
-            else:
-                new = False
-
-            db_pos.x_center = centre[0] or -999.
-            db_pos.y_center = centre[1] or -999.
-
-            db_pos.save(force_insert=new)
-
     def report_status(self):
         """Returns a dict with the position and status of each positioner."""
 
@@ -237,10 +219,6 @@ class FPS(BaseFPS):
         default one. Ignored if ``can`` is passed.
     ieb : bool or .IEB instance
         If `True`, connects the Instrument Electronics Box PLC controller.
-    qa : bool or path
-        A path to the database used to store QA information. If `True`, uses
-        the value from ``config.files.qa_database``. If `False`, does not do
-        any QA recording.
     loop : event loop or `None`
         The asyncio event loop. If `None`, uses `asyncio.get_event_loop` to
         get a valid loop.
@@ -265,20 +243,22 @@ class FPS(BaseFPS):
     """
 
     def __init__(self, can=None, layout=None, can_profile=None,
-                 ieb=None, qa=None, loop=None, engineering_mode=False):
+                 ieb=None, loop=None, engineering_mode=False):
 
         # Start file logger
         start_file_loggers(start_log=True, start_can=False)
 
         if config.__CONFIG_FILE__:
-            log.info(f'using configuration from {config.__CONFIG_FILE__}')
+            log.info(f'Using configuration from {config.__CONFIG_FILE__}')
         else:
-            log.warning('cannot find SDSSCORE or user configuration. Using default values.')
+            log.warning('Cannot find SDSSCORE or user configuration. '
+                        'Using default values.')
 
         self.engineering_mode = engineering_mode
 
         if engineering_mode:
-            warnings.warn('Engineering mode enable. Please don\'t break anything.',
+            warnings.warn('Engineering mode enable. '
+                          'Please don\'t break anything.',
                           JaegerUserWarning)
 
         self.loop = loop or asyncio.get_event_loop()
@@ -292,7 +272,9 @@ class FPS(BaseFPS):
             self.can = can
         else:
             try:
-                self.can = JaegerCAN.from_profile(can_profile, fps=self, loop=loop)
+                self.can = JaegerCAN.from_profile(can_profile,
+                                                  fps=self,
+                                                  loop=loop)
             except ConnectionRefusedError:
                 raise
 
@@ -319,16 +301,6 @@ class FPS(BaseFPS):
         else:
             raise ValueError(f'Invalid input value for ieb {ieb!r}.')
 
-        if qa is None:
-            qa = config['fps']['qa']
-
-        if qa is True:
-            self.qa_db = get_qa_database(config['files']['qa_database'])
-        elif qa is False:
-            self.qa_db = None
-        else:
-            self.qa_db = get_qa_database(qa)
-
         super().__init__(layout=layout)
 
         #: Position and status pollers
@@ -420,30 +392,31 @@ class FPS(BaseFPS):
 
         if not isinstance(command, Command):
             command_flag = CommandID(command)
-            CommandClass = command_flag.get_command()
+            CommandClass = command_flag.get_command_class()
 
             command = CommandClass(positioner_id=positioner_id,
                                    loop=self.loop, data=data, **kwargs)
 
         command_name = command.name
+        command_uid = command.command_uid
+        header = f'({command_name}, {positioner_id}, {command_uid}): '
 
         if not self.engineering_mode and self.locked:
             if command.safe or safe:
                 log.debug(f'FPS is locked but {command_name} is safe.')
             else:
                 command.cancel(silent=True)
-                raise FPSLockedError('solve the problem and unlock the FPS '
+                raise FPSLockedError('Solve the problem and unlock the FPS '
                                      'before sending commands.')
 
         elif not self.engineering_mode and command.move_command and self.moving:
             command.cancel(silent=True)
-            log.error('cannot send move command while the FPS is moving. '
-                      'Use FPS.stop_trajectory() to stop the FPS.')
-            return command
+            raise JaegerError('Cannot send move command while the '
+                              'FPS is moving. Use FPS.stop_trajectory() '
+                              'to stop the FPS.')
 
         if command.status.is_done:
-            log.error(f'({command_name}, {positioner_id}): trying to send a done command.')
-            return command
+            raise JaegerError(header + 'trying to send a done command.')
 
         command._override = override
         command._silent_on_conflict = silent_on_conflict
@@ -458,12 +431,10 @@ class FPS(BaseFPS):
 
         if not synchronous:
             self.can.command_queue.put_nowait(command)
-            log.debug(f'({command_name}, {positioner_id}): '
-                      'added command to CAN processing queue.')
+            log.debug(header + 'added command to CAN processing queue.')
         else:
             self.can._send_messages(command)
-            log.debug(f'({command_name}, {positioner_id}): '
-                      'sent command to CAN synchronously.')
+            log.debug(header + 'sent command to CAN synchronously.')
 
         return command
 
@@ -480,7 +451,8 @@ class FPS(BaseFPS):
             return
 
         if command.positioner_id not in self.positioner_to_bus:
-            log.error(f'positioner {command.positioner_id} has no assigned bus.')
+            raise JaegerError(f'Positioner {command.positioner_id} '
+                              'has no assigned bus.')
             command.finish_command(command.status.FAILED)
             return
 
@@ -507,7 +479,7 @@ class FPS(BaseFPS):
 
         """
 
-        warnings.warn('locking FPS.', JaegerUserWarning)
+        warnings.warn('Locking FPS.', JaegerUserWarning)
         self._locked = True
 
         if stop_trajectories:
@@ -521,9 +493,8 @@ class FPS(BaseFPS):
         for positioner in self.positioners.values():
             if positioner.collision and not self.engineering_mode:
                 self._locked = True
-                log.error('cannot unlock the FPS until all '
-                          'the collisions have been cleared.')
-                return False
+                raise JaegerError('Cannot unlock the FPS until all '
+                                  'the collisions have been cleared.')
 
         self._locked = False
 
@@ -570,21 +541,22 @@ class FPS(BaseFPS):
         else:
             n_expected_positioners = None
 
-        get_firmware_command = self.send_command(CommandID.GET_FIRMWARE_VERSION,
-                                                 positioner_id=0,
-                                                 timeout=config['fps']['initialise_timeouts'],
-                                                 n_positioners=n_expected_positioners)
+        get_firmware_command = self.send_command(
+            CommandID.GET_FIRMWARE_VERSION,
+            positioner_id=0,
+            timeout=config['fps']['initialise_timeouts'],
+            n_positioners=n_expected_positioners)
 
         await get_firmware_command
 
         if get_firmware_command.status.failed:
             if not self.engineering_mode:
-                log.error('failed retrieving firmware version. '
-                          'Cannot initialise FPS.')
-                return False
+                raise JaegerError('Failed retrieving firmware version. '
+                                  'Cannot initialise FPS.')
             else:
-                warnings.warn('failed retrieving firmware version. '
-                              'Continuing because engineering mode.', JaegerUserWarning)
+                warnings.warn('Failed retrieving firmware version. '
+                              'Continuing because engineering mode.',
+                              JaegerUserWarning)
 
         # Loops over each reply and set the positioner status to OK. If the
         # positioner was not in the list, adds it. Checks how many positioner
@@ -598,16 +570,16 @@ class FPS(BaseFPS):
                     unknwon_positioners.append(positioner_id)
                     self.add_positioner(positioner_id)
                 else:
-                    log.error(f'found positioner with ID={positioner_id} '
-                              'that is not in the layout.')
-                    return False
+                    raise JaegerError('Found positioner with '
+                                      f'ID={positioner_id} '
+                                      'that is not in the layout.')
 
             positioner = self.positioners[positioner_id]
             positioner.firmware = get_firmware_command.get_firmware(positioner_id)
 
         if len(set([pos.firmware for pos in self.values()])) > 1:
-            warnings.warn('positioners with different firmware versions found.',
-                          JaegerUserWarning)
+            warnings.warn('Positioners with different firmware '
+                          'versions found.', JaegerUserWarning)
 
         # Stop positioners that are not in bootloader mode.
         await self.stop_trajectory()
@@ -615,9 +587,10 @@ class FPS(BaseFPS):
         await self.update_status(timeout=config['fps']['initialise_timeouts'])
 
         if len(unknwon_positioners) > 0:
-            warnings.warn(f'found {len(unknwon_positioners)} unknown positioners '
+            warnings.warn(f'Found {len(unknwon_positioners)} unknown positioners '
                           f'with IDs {sorted(unknwon_positioners)!r}. '
-                          'They have been added to the layout.', JaegerUserWarning)
+                          'They have been added to the layout.',
+                          JaegerUserWarning)
 
         n_did_not_reply = len([pos for pos in self.positioners
                                if self[pos].status == self[pos].flags.UNKNOWN])
@@ -631,14 +604,14 @@ class FPS(BaseFPS):
                                      not self[pos].initialised)])
 
         if n_non_initialised > 0:
-            warnings.warn(f'{n_non_initialised} positioners responded but have '
-                          'not been initialised.', JaegerUserWarning)
+            warnings.warn(f'{n_non_initialised} positioners responded but '
+                          'have not been initialised.', JaegerUserWarning)
 
         if self.locked:
             log.info('FPS is locked. Trying to unlock it.')
             if not await self.unlock():
-                log.error('FPS cannot be unlocked. Initialisation failed.')
-                return False
+                raise JaegerError('FPS cannot be unlocked. '
+                                  'Initialisation failed.')
             else:
                 log.info('FPS unlocked successfully.')
 
@@ -649,10 +622,12 @@ class FPS(BaseFPS):
         results = await asyncio.gather(*initialise_cmds)
 
         if False in results:
-            log.error('some positioners failed to initialise.')
             if self.engineering_mode:
-                warnings.warn('continuing because engineering mode ...',
+                warnings.warn('Some positioners failed to initialise. '
+                              'Continuing because engineering mode ...',
                               JaegerUserWarning)
+            else:
+                raise JaegerError('Some positioners failed to initialise.')
 
         await self.update_position()
 
@@ -681,7 +656,7 @@ class FPS(BaseFPS):
 
         if positioner_ids:
             n_positioners = len(positioner_ids)
-        elif positioner_ids is None:
+        else:
             # This is the max number that should reply.
             n_positioners = len(self) if len(self) > 0 else None
 
@@ -695,7 +670,7 @@ class FPS(BaseFPS):
         await command
 
         if command.status.failed:
-            log.warning(f'failed broadcasting {CommandID.GET_STATUS.name!r} '
+            log.warning(f'Failed broadcasting {CommandID.GET_STATUS.name!r} '
                         'during update status.')
             return False
 
@@ -703,7 +678,8 @@ class FPS(BaseFPS):
         for reply in command.replies:
 
             pid = reply.positioner_id
-            if pid not in self.positioners or (positioner_ids and pid not in positioner_ids):
+            if pid not in self.positioners or (positioner_ids and
+                                               pid not in positioner_ids):
                 continue
 
             positioner = self.positioners[pid]
@@ -759,8 +735,8 @@ class FPS(BaseFPS):
                 position = command.get_positions()
                 update_position_commands.append(self[pid].update_position(position))
             except ValueError as ee:
-                log.error(f'failed updating position for positioner {pid}: {ee}')
-                return False
+                raise JaegerError('Failed updating position for '
+                                  f'positioner {pid}: {ee}')
 
         await asyncio.gather(*update_position_commands)
 
@@ -796,12 +772,12 @@ class FPS(BaseFPS):
         await get_firmware_command
 
         if get_firmware_command.status.failed:
-            log.error('failed retrieving firmware version.')
-            return False
+            raise JaegerError('Failed retrieving firmware version.')
 
         for reply in get_firmware_command.replies:
             pid = reply.positioner_id
-            if pid not in self.positioners or (positioner_ids and pid not in positioner_ids):
+            if pid not in self.positioners or (positioner_ids and
+                                               pid not in positioner_ids):
                 continue
 
             positioner = self.positioners[pid]
@@ -832,10 +808,13 @@ class FPS(BaseFPS):
             if positioners == []:
                 return
 
-        await self.send_to_all('TRAJECTORY_TRANSMISSION_ABORT', positioners=positioners)
+        await self.send_to_all('TRAJECTORY_TRANSMISSION_ABORT',
+                               positioners=positioners)
 
         if clear_flags:
-            await self.send_command('STOP_TRAJECTORY', positioner_id=0, timeout=timeout)
+            await self.send_command('STOP_TRAJECTORY',
+                                    positioner_id=0,
+                                    timeout=timeout)
 
     async def send_trajectory(self, *args, **kwargs):
         """Sends a set of trajectories to the positioners.
@@ -847,8 +826,7 @@ class FPS(BaseFPS):
         try:
             return await send_trajectory(self, *args, **kwargs)
         except TrajectoryError as ee:
-            log.error(f'sending trajectory failed with error: {ee}')
-            return False
+            raise JaegerError(f'Sending trajectory failed with error: {ee}')
 
     def abort(self):
         """Aborts trajectories and stops positioners."""
@@ -885,7 +863,9 @@ class FPS(BaseFPS):
         positioners = positioners or list(self.positioners.keys())
 
         if data is None or len(data) == 1:
-            commands = [self.send_command(command, positioner_id=positioner_id, **kwargs)
+            commands = [self.send_command(command,
+                                          positioner_id=positioner_id,
+                                          **kwargs)
                         for positioner_id in positioners]
         else:
             commands = [self.send_command(command, positioner_id=positioner_id,
@@ -895,8 +875,8 @@ class FPS(BaseFPS):
         results = await asyncio.gather(*commands, return_exceptions=True)
 
         if any([isinstance(rr, FPSLockedError) for rr in results]):
-            raise FPSLockedError('one or more of the commands failed because '
-                                 'the FPS is locked.')
+            raise FPSLockedError('One or more of the commands failed '
+                                 'because the FPS is locked.')
 
         return commands
 
@@ -907,15 +887,15 @@ class FPS(BaseFPS):
                           for positioner in self.values()])
 
         if not bootloader:
-            log.info('stopping positioners')
+            log.info('Stopping positioners')
             await self.stop_trajectory()
 
-        log.info('stopping all pollers.')
+        log.info('Stopping all pollers.')
         await self.pollers.stop()
 
         await asyncio.sleep(1)
 
-        log.info('cancelling all pending tasks and shutting down.')
+        log.info('Cancelling all pending tasks and shutting down.')
 
         tasks = [task for task in asyncio.all_tasks(loop=self.loop)
                  if task is not asyncio.current_task(loop=self.loop)]
